package com.qmth.sop.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.sop.business.bean.params.FlowApproveParam;
import com.qmth.sop.business.bean.params.SopInfoListParam;
import com.qmth.sop.business.bean.params.SopPublishParam;
import com.qmth.sop.business.bean.params.SopSaveParam;
import com.qmth.sop.business.bean.result.CrmProjectResult;
import com.qmth.sop.business.bean.result.FlowResult;
import com.qmth.sop.business.bean.result.FlowTaskResult;
import com.qmth.sop.business.bean.result.SopInfoResult;
import com.qmth.sop.business.entity.TBSopInfo;
import com.qmth.sop.business.entity.TFCustomFlow;
import com.qmth.sop.business.entity.TFCustomFlowEntity;
import com.qmth.sop.common.enums.SopStatusEnum;
import com.qmth.sop.common.enums.TFCustomTypeEnum;

import java.util.List;
import java.util.Map;

/**
 * <p>
 * sop表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBSopInfoService extends IService<TBSopInfo> {

    /**
     * 查询动态sop表名是否存在
     *
     * @param db
     * @param tableName
     * @return
     */
    int selectSopDynamicExist(String db, String tableName);

    /**
     * 保存动态sop信息
     *
     * @param tfCustomFlow
     * @param code
     * @param flowId
     * @param flowTaskResult
     * @return
     */
    Boolean saveDynamicSop(TFCustomFlow tfCustomFlow, String code, Long flowId, FlowTaskResult flowTaskResult);

    /**
     * 查询动态sop数据
     *
     * @param tableName
     * @param flowDeploymentId
     * @param flowId
     * @return
     */
    int selectDynamicSop(String tableName, String flowDeploymentId, Long flowId);

    /**
     * sop发布信息
     *
     * @param sopPublishParam
     * @return
     */
    String sopPublish(SopPublishParam sopPublishParam) throws Exception;

    /**
     * sop填报信息
     *
     * @param flowApproveParam
     * @return
     */
    Boolean sopApprove(FlowApproveParam flowApproveParam) throws InterruptedException;

    /**
     * sop管理列表
     *
     * @param sopInfoListParam
     * @return
     */
    IPage<Map> list(SopInfoListParam sopInfoListParam);

    /**
     * 根据crmNo查询在途的sop信息
     *
     * @param crmNo
     * @return
     */
    List<String> findFlowPassageByCrmNo(String crmNo);

    /**
     * 根据serviceId查询所有sop信息
     *
     * @param serviceId
     * @return
     */
    List<SopInfoResult> findFlowByServiceId(Long serviceId);

    /**
     * 保存提醒/延时任务
     *
     * @param map
     */
    public void saveJobRemind(Map<String, Object> map) throws InterruptedException;

    /**
     * 更新sop计划进场开始时间和结束时间
     *
     * @param flowTaskResult
     * @param sopId
     * @param flowId
     * @param userId
     * @param type
     * @throws InterruptedException
     */
    public void saveSopPlanDate(FlowTaskResult flowTaskResult, Long sopId, Long flowId, Long userId,
            TFCustomTypeEnum type) throws InterruptedException;

    /**
     * sop编辑
     *
     * @param id
     * @return
     */
    public FlowResult sopEdit(Long id) throws InterruptedException;

    /**
     * sop保存
     *
     * @param sopSaveParam
     * @return
     */
    public Boolean sopSave(SopSaveParam sopSaveParam) throws InterruptedException;

    /**
     * 获取表单内容
     *
     * @param tfCustomFlowEntity
     * @param setup
     * @return
     */
    public FlowTaskResult getFormProperties(TFCustomFlowEntity tfCustomFlowEntity, Integer setup);

    /**
     * 设置流程第一步值
     *
     * @param sopPublishParam
     * @param tfCustomFlow
     * @param crmProjectResult
     * @return
     * @throws Exception
     */
    public SopPublishParam setSetupOne(SopPublishParam sopPublishParam, TFCustomFlow tfCustomFlow,
            CrmProjectResult crmProjectResult) throws Exception;

    /**
     * sop转审
     *
     * @param crmDetailId
     * @return
     */
    Boolean sopApproverExchange(Long crmDetailId) throws InterruptedException;

    /**
     * 查询sop状态
     *
     * @param sopNo
     * @return
     */
    SopStatusEnum findBySopNo(String sopNo);

    /**
     * sop人员配置
     *
     * @param flowId
     * @return
     */
    Boolean sopAllocation(Long flowId);

    /**
     * 修改流程第一步
     *
     * @param flowId
     * @param crmDetailId
     * @return
     */
    Boolean updateFlowSetupOne(Long flowId, Long crmDetailId);
}
